@@ -2,6 +2,7 @@
 
 /*
  * Copyright (C) 2002 Havoc Pennington
+ * Copyright (C) 2002 Mathias Hasselmann
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,12 +22,15 @@
 
 #include "profile-editor.h"
 #include "terminal-intl.h"
+#include "terminal.h"
 #include <glade/glade.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-color-picker.h>
 #include <libgnomeui/gnome-font-picker.h>
 #include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-icon-entry.h>
 #include <string.h>
+#include <math.h>
 #include "simple-x-font-selector.h"
 #include "terminal-widget.h"
 
@@ -40,16 +44,19 @@ struct _TerminalColorScheme
   GdkColor foreground;
   GdkColor background;
 };
-  
+
+/* alphabetized */
 static TerminalColorScheme color_schemes[] = {
   { N_("Black on light yellow"),
     { 0, 0x0000, 0x0000, 0x0000 }, { 0, 0xFFFF, 0xFFFF, 0xDDDD } },
-  { N_("White on black"),
-    { 0, 0xFFFF, 0xFFFF, 0xFFFF }, { 0, 0x0000, 0x0000, 0x0000 } },
   { N_("Black on white"),
     { 0, 0x0000, 0x0000, 0x0000 }, { 0, 0xFFFF, 0xFFFF, 0xFFFF } },
+  { N_("Gray on black"),
+    { 0, 0xAAAA, 0xAAAA, 0xAAAA }, { 0, 0x0000, 0x0000, 0x0000 } },
   { N_("Green on black"),
-    { 0, 0x0000, 0xFFFF, 0x0000 }, { 0, 0x0000, 0x0000, 0x0000 } }  
+    { 0, 0x0000, 0xFFFF, 0x0000 }, { 0, 0x0000, 0x0000, 0x0000 } },
+  { N_("White on black"),
+    { 0, 0xFFFF, 0xFFFF, 0xFFFF }, { 0, 0x0000, 0x0000, 0x0000 } }
 };
 
 typedef struct _TerminalPaletteScheme TerminalPaletteScheme;
@@ -135,11 +142,12 @@ static void       profile_editor_update_font                 (GtkWidget       *w
                                                               TerminalProfile *profile);
 
 
-static void profile_forgotten (TerminalProfile     *profile,
-                               GtkWidget           *editor);
-static void profile_changed   (TerminalProfile     *profile,
-                               TerminalSettingMask  mask,
-                               GtkWidget           *editor);
+static void profile_forgotten (TerminalProfile           *profile,
+                               GtkWidget                 *editor);
+static void profile_changed   (TerminalProfile           *profile,
+                               const TerminalSettingMask *mask,
+                               GtkWidget                 *editor);
+
 
 
 static void
@@ -243,102 +251,102 @@ profile_forgotten (TerminalProfile *profile,
 }
 
 static void
-profile_changed (TerminalProfile          *profile,
-                 TerminalSettingMask       mask,
-                 GtkWidget                *editor)
+profile_changed (TerminalProfile           *profile,
+                 const TerminalSettingMask *mask,
+                 GtkWidget                 *editor)
 {
-  if (mask & TERMINAL_SETTING_VISIBLE_NAME)
+  if (mask->visible_name)
     profile_editor_update_visible_name (editor, profile);
 
-  if (mask & TERMINAL_SETTING_ICON)
+  if (mask->icon_file)
     profile_editor_update_icon (editor, profile);
   
-  if (mask & TERMINAL_SETTING_CURSOR_BLINK)
+  if (mask->cursor_blink)
     profile_editor_update_cursor_blink (editor, profile);
 
-  if (mask & TERMINAL_SETTING_DEFAULT_SHOW_MENUBAR)
+  if (mask->default_show_menubar)
     profile_editor_update_default_show_menubar (editor, profile);
 
-  if ((mask & TERMINAL_SETTING_FOREGROUND_COLOR) ||
-      (mask & TERMINAL_SETTING_BACKGROUND_COLOR))
+  if ((mask->foreground_color) ||
+      (mask->background_color))
     {
       profile_editor_update_color_scheme_menu (editor, profile);
       profile_editor_update_color_pickers (editor, profile);
     }
 
-  if (mask & TERMINAL_SETTING_TITLE)
+  if (mask->title)
     profile_editor_update_title (editor, profile);
   
-  if (mask & TERMINAL_SETTING_TITLE_MODE)
+  if (mask->title_mode)
     profile_editor_update_title_mode (editor, profile);
 
-  if (mask & TERMINAL_SETTING_ALLOW_BOLD)
+  if (mask->allow_bold)
     profile_editor_update_allow_bold (editor, profile);
 
-  if (mask & TERMINAL_SETTING_SILENT_BELL)
+  if (mask->silent_bell)
     profile_editor_update_silent_bell (editor, profile);
 
-  if (mask & TERMINAL_SETTING_WORD_CHARS)
+  if (mask->word_chars)
     profile_editor_update_word_chars (editor, profile);
 
-  if (mask & TERMINAL_SETTING_SCROLLBAR_POSITION)
+  if (mask->scrollbar_position)
     profile_editor_update_scrollbar_position (editor, profile);
 
-  if (mask & TERMINAL_SETTING_SCROLLBACK_LINES)
+  if (mask->scrollback_lines)
     profile_editor_update_scrollback_lines (editor, profile);
 
-  if (mask & TERMINAL_SETTING_SCROLL_ON_KEYSTROKE)
+  if (mask->scroll_on_keystroke)
     profile_editor_update_scroll_on_keystroke (editor, profile);
 
-  if (mask & TERMINAL_SETTING_SCROLL_ON_OUTPUT)
+  if (mask->scroll_on_output)
     profile_editor_update_scroll_on_output (editor, profile);
 
-  if (mask & TERMINAL_SETTING_EXIT_ACTION)
+  if (mask->exit_action)
     profile_editor_update_exit_action (editor, profile);
 
-  if (mask & TERMINAL_SETTING_LOGIN_SHELL)
+  if (mask->login_shell)
     profile_editor_update_login_shell (editor, profile);
 
-  if (mask & TERMINAL_SETTING_UPDATE_RECORDS)
+  if (mask->update_records)
     profile_editor_update_update_records (editor, profile);
 
-  if (mask & TERMINAL_SETTING_USE_CUSTOM_COMMAND)
+  if (mask->use_custom_command)
     profile_editor_update_use_custom_command (editor, profile);
 
-  if (mask & TERMINAL_SETTING_CUSTOM_COMMAND)
+  if (mask->custom_command)
     profile_editor_update_custom_command (editor, profile);
 
-  if (mask & TERMINAL_SETTING_PALETTE)
+  if (mask->palette)
     profile_editor_update_palette (editor, profile);
 
-  if (mask & TERMINAL_SETTING_X_FONT)
+  if (mask->x_font)
     profile_editor_update_x_font (editor, profile);
 
-  if (mask & TERMINAL_SETTING_BACKGROUND_TYPE)
+  if (mask->background_type)
     profile_editor_update_background_type (editor, profile);
   
-  if (mask & TERMINAL_SETTING_BACKGROUND_IMAGE)
+  if (mask->background_image_file)
     profile_editor_update_background_image (editor, profile);
 
-  if (mask & TERMINAL_SETTING_SCROLL_BACKGROUND)
+  if (mask->scroll_background)
     profile_editor_update_scroll_background (editor, profile);
 
-  if (mask & TERMINAL_SETTING_BACKGROUND_DARKNESS)
+  if (mask->background_darkness)
     profile_editor_update_background_darkness (editor, profile);
 
-  if (mask & TERMINAL_SETTING_BACKSPACE_BINDING)
+  if (mask->backspace_binding)
     profile_editor_update_backspace_binding (editor, profile);
 
-  if (mask & TERMINAL_SETTING_DELETE_BINDING)
+  if (mask->delete_binding)
     profile_editor_update_delete_binding (editor, profile);
 
-  if (mask & TERMINAL_SETTING_USE_THEME_COLORS)
+  if (mask->use_theme_colors)
     profile_editor_update_use_theme_colors (editor, profile);
     
-  if (mask & TERMINAL_SETTING_USE_SYSTEM_FONT)
+  if (mask->use_system_font)
     profile_editor_update_use_system_font (editor, profile);
 
-  if (mask & TERMINAL_SETTING_FONT)
+  if (mask->font)
     profile_editor_update_font (editor, profile);
   
   profile_editor_update_sensitivity (editor, profile);
@@ -476,11 +484,11 @@ allow_bold_toggled (GtkWidget       *checkbutton,
 }
 
 static void
-silent_bell_toggled (GtkWidget       *checkbutton,
-                     TerminalProfile *profile)
+bell_toggled (GtkWidget       *checkbutton,
+              TerminalProfile *profile)
 {
   terminal_profile_set_silent_bell (profile,
-                                    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
+                                    !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)));
 }
 
 static void
@@ -692,12 +700,18 @@ scroll_background_toggled (GtkWidget       *checkbutton,
 }
 
 static void
-darken_background_toggled (GtkWidget       *checkbutton,
-                           TerminalProfile *profile)
+darken_background_value_changed (GtkWidget       *scale,
+                                 TerminalProfile *profile)
 {
-  terminal_profile_set_background_darkness (profile,
-                                            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton)) ?
-                                            DARKNESS_TRUE : DARKNESS_FALSE);
+  double new_val;
+  double old_val;
+
+  new_val = gtk_range_get_value (GTK_RANGE (scale));
+  old_val = terminal_profile_get_background_darkness (profile);
+
+  /* The epsilon here is some anti-infinite-loop paranoia */
+  if (fabs (new_val - old_val) > 1e-6)
+    terminal_profile_set_background_darkness (profile, new_val);
 }
 
 static void
@@ -835,6 +849,48 @@ init_palette_scheme_menu (GtkWidget *option_menu)
                             menu);
 }
 
+static char*
+format_percent_value (GtkScale *scale,
+                      double    val,
+                      void     *data)
+{
+  return g_strdup_printf ("%d%%", (int) rint (val * 100.0));
+}
+
+static void
+init_background_darkness_scale (GtkWidget *scale)
+{
+  g_signal_connect (G_OBJECT (scale), "format_value",
+                    G_CALLBACK (format_percent_value),
+                    NULL);
+}
+
+
+static void
+editor_response_cb (GtkDialog *editor,
+                    int        id,
+                    void      *data)
+{  
+  if (id == GTK_RESPONSE_HELP)
+    {
+      GError *err;
+      err = NULL;
+      gnome_help_display ("gnome-terminal", "gnome-terminal-prefs",
+                          &err);
+      
+      if (err)
+        {
+          terminal_util_show_error_dialog (GTK_WINDOW (editor), NULL,
+                                           _("There was an error displaying help: %s"), err->message);
+          g_error_free (err);
+        }
+    }
+  else
+    {
+      gtk_widget_destroy (GTK_WIDGET (editor));
+    }
+}
+
 void
 terminal_profile_edit (TerminalProfile *profile,
                        GtkWindow       *transient_parent)
@@ -852,49 +908,12 @@ terminal_profile_edit (TerminalProfile *profile,
       double num1, num2;
       gint i;
       GtkSizeGroup *size_group;
-      
-      if (g_file_test ("./"TERM_GLADE_FILE,
-                       G_FILE_TEST_EXISTS))
-        {
-          /* Try current dir, for debugging */
-          xml = glade_xml_new ("./"TERM_GLADE_FILE,
-                               "profile-editor-dialog",
-                               GETTEXT_PACKAGE);
-        }
-      else
-        {
-          xml = glade_xml_new (TERM_GLADE_DIR"/"TERM_GLADE_FILE,
-                               "profile-editor-dialog",
-                               GETTEXT_PACKAGE);
-        }
 
+      xml = terminal_util_load_glade_file (TERM_GLADE_FILE,
+                                           "profile-editor-dialog",
+                                           transient_parent);
       if (xml == NULL)
-        {
-          static GtkWidget *no_glade_dialog = NULL;
-          
-          if (no_glade_dialog == NULL)
-            {
-              no_glade_dialog =
-                gtk_message_dialog_new (transient_parent,
-                                        GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_MESSAGE_ERROR,
-                                        GTK_BUTTONS_CLOSE,
-                                        _("The file \"%s\" is missing. This indicates that the application is installed incorrectly, so the profile editor can't be displayed."),
-                                        TERM_GLADE_DIR"/"TERM_GLADE_FILE);
-                                        
-              g_signal_connect (G_OBJECT (no_glade_dialog),
-                                "response",
-                                G_CALLBACK (gtk_widget_destroy),
-                                NULL);
-
-              g_object_add_weak_pointer (G_OBJECT (no_glade_dialog),
-                                         (void**)&no_glade_dialog);
-            }
-
-          gtk_window_present (GTK_WINDOW (no_glade_dialog));
-
-          return;
-        }
+        return;
       
       old_transient_parent = NULL;
       
@@ -905,6 +924,7 @@ terminal_profile_edit (TerminalProfile *profile,
       gtk_dialog_set_has_separator (GTK_DIALOG (editor), FALSE);
       
       gtk_dialog_add_buttons (GTK_DIALOG (editor),
+                              GTK_STOCK_HELP, GTK_RESPONSE_HELP,
                               GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT,
                               NULL);
       gtk_dialog_set_default_response (GTK_DIALOG (editor), GTK_RESPONSE_ACCEPT);
@@ -928,7 +948,7 @@ terminal_profile_edit (TerminalProfile *profile,
 
       g_signal_connect (G_OBJECT (editor),
                         "response",
-                        G_CALLBACK (gtk_widget_destroy),
+                        G_CALLBACK (editor_response_cb),
                         NULL);
       
       gtk_window_set_destroy_with_parent (GTK_WINDOW (editor), TRUE);
@@ -1008,10 +1028,10 @@ terminal_profile_edit (TerminalProfile *profile,
                         G_CALLBACK (allow_bold_toggled),
                         profile);
       
-      w = glade_xml_get_widget (xml, "silent-bell-checkbutton");
+      w = glade_xml_get_widget (xml, "bell-checkbutton");
       profile_editor_update_silent_bell (editor, profile);
       g_signal_connect (G_OBJECT (w), "toggled",
-                        G_CALLBACK (silent_bell_toggled),
+                        G_CALLBACK (bell_toggled),
                         profile);
 
       w = glade_xml_get_widget (xml, "word-chars-entry");
@@ -1026,12 +1046,15 @@ terminal_profile_edit (TerminalProfile *profile,
                         G_CALLBACK (scrollbar_position_changed),
                         profile);
 
+      size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+      
       w = glade_xml_get_widget (xml, "scrollback-lines-spinbutton");
       profile_editor_update_scrollback_lines (editor, profile);
       g_signal_connect (G_OBJECT (w), "value_changed",
                         G_CALLBACK (scrollback_lines_value_changed),
                         profile);
-
+      gtk_size_group_add_widget (size_group, w);
+      
       gtk_spin_button_get_range (GTK_SPIN_BUTTON (w), &num1, &num2);
 
       w = glade_xml_get_widget (xml, "scrollback-kilobytes-spinbutton");
@@ -1045,6 +1068,9 @@ terminal_profile_edit (TerminalProfile *profile,
       g_signal_connect (G_OBJECT (w), "value_changed",
                         G_CALLBACK (scrollback_kilobytes_value_changed),
                         profile);
+      gtk_size_group_add_widget (size_group, w);
+
+      g_object_unref (G_OBJECT (size_group));
       
       w = glade_xml_get_widget (xml, "scroll-on-keystroke-checkbutton");
       profile_editor_update_scroll_on_keystroke (editor, profile);
@@ -1121,10 +1147,11 @@ terminal_profile_edit (TerminalProfile *profile,
                         G_CALLBACK (scroll_background_toggled),
                         profile);
       
-      w = glade_xml_get_widget (xml, "darken-background-checkbutton");
+      w = glade_xml_get_widget (xml, "darken-background-scale");
+      init_background_darkness_scale (w);
       profile_editor_update_background_darkness (editor, profile);
-      g_signal_connect (G_OBJECT (w), "toggled",
-                        G_CALLBACK (darken_background_toggled),
+      g_signal_connect (G_OBJECT (w), "value_changed",
+                        G_CALLBACK (darken_background_value_changed),
                         profile);
 
       w = glade_xml_get_widget (xml, "backspace-binding-optionmenu");
@@ -1145,6 +1172,13 @@ terminal_profile_edit (TerminalProfile *profile,
                         G_CALLBACK (use_theme_colors_toggled),
                         profile);
       
+      w = glade_xml_get_widget (xml, "system-font-checkbutton");
+      profile_editor_update_use_system_font (editor, profile);
+      g_signal_connect (G_OBJECT (w), "toggled",
+                        G_CALLBACK (use_system_font_toggled),
+                        profile);
+
+      
       i = 0;
       while (i < TERMINAL_PALETTE_SIZE)
         {
@@ -1172,6 +1206,8 @@ terminal_profile_edit (TerminalProfile *profile,
 
       if (terminal_widget_supports_pango_fonts ())
         {
+          GtkWidget *font_label;
+          
           fontsel = gnome_font_picker_new ();
           g_object_set_data (G_OBJECT (editor), "font-selector", fontsel);
 
@@ -1185,6 +1221,25 @@ terminal_profile_edit (TerminalProfile *profile,
           g_signal_connect (G_OBJECT (fontsel), "font_set",
                             G_CALLBACK (font_set),
                             profile);
+
+          font_label = gtk_label_new_with_mnemonic (_("_Font:"));
+          gtk_misc_set_alignment (GTK_MISC (font_label), 0.0, 0.5);
+
+          gtk_box_set_spacing (GTK_BOX (w), 12);
+          
+          gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (font_label), FALSE, FALSE, 0);
+          gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), FALSE, FALSE, 0);
+
+          size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+          gtk_size_group_add_widget (size_group,
+                                     font_label);
+          gtk_size_group_add_widget (size_group,
+                                     glade_xml_get_widget (xml,
+                                                           "profile-name-label"));
+          gtk_size_group_add_widget (size_group,
+                                     glade_xml_get_widget (xml,
+                                                           "profile-icon-label"));
+          g_object_unref (G_OBJECT (size_group));
         }
       else
         {
@@ -1206,9 +1261,10 @@ terminal_profile_edit (TerminalProfile *profile,
                                      glade_xml_get_widget (xml,
                                                            "profile-icon-label"));
           g_object_unref (G_OBJECT (size_group));
+
+          gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), TRUE, TRUE, 0);
         }
-          
-      gtk_box_pack_start (GTK_BOX (w), GTK_WIDGET (fontsel), TRUE, TRUE, 0);
+      
       gtk_widget_show_all (w);
       
       w = glade_xml_get_widget (xml, "reset-compat-defaults-button");
@@ -1248,16 +1304,19 @@ static void
 profile_editor_update_sensitivity (GtkWidget       *editor,
                                    TerminalProfile *profile)
 {
-  TerminalSettingMask mask;
-  TerminalSettingMask last_mask;
+  const TerminalSettingMask *mask;
   GtkWidget *w;
   
   /* the first time in this function the object data is unset
    * thus the last mask is 0, which means everything is sensitive,
    * which is what we want.
    */
+  /* disabled for now for testing */
+#if 0
+  TerminalSettingMask last_mask;
   last_mask = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (editor),
                                                   "cached-lock-mask"));
+#endif
   
   mask = terminal_profile_get_locked_settings (profile);
 
@@ -1266,14 +1325,26 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
    */
   w = profile_editor_get_widget (editor, "custom-command-entry");
   gtk_widget_set_sensitive (w,
-                            !((mask & TERMINAL_SETTING_CUSTOM_COMMAND) ||
+                            !((mask->custom_command) ||
                               !terminal_profile_get_use_custom_command (profile))); 
   if (terminal_profile_get_background_type (profile) == TERMINAL_BACKGROUND_IMAGE)
     {
       w = profile_editor_get_widget (editor, "background-image-fileentry");
-      gtk_widget_set_sensitive (w, !(mask & TERMINAL_SETTING_BACKGROUND_IMAGE));
+      gtk_widget_set_sensitive (w, !(mask->background_image_file));
       w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
-      gtk_widget_set_sensitive (w, !(mask & TERMINAL_SETTING_SCROLL_BACKGROUND));
+      gtk_widget_set_sensitive (w, !(mask->scroll_background));
+      w = profile_editor_get_widget (editor, "darken-background-vbox");
+      gtk_widget_set_sensitive (w, !(mask->background_darkness));
+    }
+  else if (terminal_profile_get_background_type (profile) == TERMINAL_BACKGROUND_TRANSPARENT) 
+  {
+      w = profile_editor_get_widget (editor, "darken-background-vbox");
+      gtk_widget_set_sensitive (w, !(mask->background_darkness));
+      w = profile_editor_get_widget (editor, "background-image-fileentry");
+      gtk_widget_set_sensitive (w, FALSE);
+      w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
+      gtk_widget_set_sensitive (w, FALSE);
+      
     }
   else
     {
@@ -1281,8 +1352,24 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
       gtk_widget_set_sensitive (w, FALSE);
       w = profile_editor_get_widget (editor, "scroll-background-checkbutton");
       gtk_widget_set_sensitive (w, FALSE);
+      w = profile_editor_get_widget (editor, "darken-background-vbox");
+      gtk_widget_set_sensitive (w, FALSE);
     }
 
+  if (!terminal_profile_get_use_system_font (profile))
+    {
+      if (terminal_widget_supports_pango_fonts ())
+        set_insensitive (editor, "font-hbox", mask->font);
+      else
+        set_insensitive (editor, "font-hbox", mask->x_font);
+    }
+  else
+    {
+      w = profile_editor_get_widget (editor, "font-hbox");
+      gtk_widget_set_sensitive (w, FALSE);
+    }
+
+
   if (terminal_profile_get_use_theme_colors (profile))
     {
       set_insensitive (editor, "foreground-colorpicker", TRUE);
@@ -1292,105 +1379,98 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
   else
     {      
       set_insensitive (editor, "foreground-colorpicker",
-                       mask & TERMINAL_SETTING_FOREGROUND_COLOR);
+                       mask->foreground_color);
       
       set_insensitive (editor, "background-colorpicker",
-                       mask & TERMINAL_SETTING_BACKGROUND_COLOR);
+                       mask->background_color);
       
       set_insensitive (editor, "color-scheme-optionmenu",
-                       (mask & (TERMINAL_SETTING_BACKGROUND_COLOR |
-                                TERMINAL_SETTING_FOREGROUND_COLOR)));
+                       (mask->background_color) ||
+                       (mask->foreground_color));
     }
   
 #if 0 /* uncomment once we've tested the sensitivity code */
   if (mask == last_mask)
     return;
-#endif
-
   g_object_set_data (G_OBJECT (editor), "cached-lock-mask",
                      GINT_TO_POINTER (mask));
+#endif
   
   set_insensitive (editor, "profile-name-entry",
-                   mask & TERMINAL_SETTING_VISIBLE_NAME);
+                   mask->visible_name);
 
   set_insensitive (editor, "profile-icon-entry",
-                   mask & TERMINAL_SETTING_ICON);
+                   mask->icon_file);
   
   set_insensitive (editor, "blink-cursor-checkbutton",
-                   mask & TERMINAL_SETTING_CURSOR_BLINK);
+                   mask->cursor_blink);
 
   set_insensitive (editor, "show-menubar-checkbutton",
-                   mask & TERMINAL_SETTING_DEFAULT_SHOW_MENUBAR);
+                   mask->default_show_menubar);
 
   set_insensitive (editor, "title-entry",
-                   mask & TERMINAL_SETTING_TITLE);
+                   mask->title);
   
   set_insensitive (editor, "title-mode-optionmenu",
-                   mask & TERMINAL_SETTING_TITLE_MODE);
+                   mask->title_mode);
 
   set_insensitive (editor, "allow-bold-checkbutton",
-                   mask & TERMINAL_SETTING_ALLOW_BOLD);
+                   mask->allow_bold);
 
-  set_insensitive (editor, "silent-bell-checkbutton",
-                   mask & TERMINAL_SETTING_SILENT_BELL);
+  set_insensitive (editor, "bell-checkbutton",
+                   mask->silent_bell);
 
   set_insensitive (editor, "word-chars-entry",
-                   mask & TERMINAL_SETTING_WORD_CHARS);
+                   mask->word_chars);
 
   set_insensitive (editor, "scrollbar-position-optionmenu",
-                   mask & TERMINAL_SETTING_SCROLLBAR_POSITION);
+                   mask->scrollbar_position);
 
   set_insensitive (editor, "scrollback-lines-spinbutton",
-                   mask & TERMINAL_SETTING_SCROLLBACK_LINES);
+                   mask->scrollback_lines);
 
   set_insensitive (editor, "scrollback-kilobytes-spinbutton",
-                   mask & TERMINAL_SETTING_SCROLLBACK_LINES);
+                   mask->scrollback_lines);
   
   set_insensitive (editor, "scroll-on-keystroke-checkbutton",
-                   mask & TERMINAL_SETTING_SCROLL_ON_KEYSTROKE);
+                   mask->scroll_on_keystroke);
 
   set_insensitive (editor, "scroll-on-output-checkbutton",
-                   mask & TERMINAL_SETTING_SCROLL_ON_OUTPUT);
+                   mask->scroll_on_output);
 
   set_insensitive (editor, "exit-action-optionmenu",
-                   mask & TERMINAL_SETTING_EXIT_ACTION);
+                   mask->exit_action);
 
   set_insensitive (editor, "login-shell-checkbutton",
-                   mask & TERMINAL_SETTING_LOGIN_SHELL);
+                   mask->login_shell);
 
   set_insensitive (editor, "update-records-checkbutton",
-                   mask & TERMINAL_SETTING_UPDATE_RECORDS);
+                   mask->update_records);
 
   set_insensitive (editor, "use-custom-command-checkbutton",
-                   mask & TERMINAL_SETTING_USE_CUSTOM_COMMAND);
+                   mask->use_custom_command);
 
   set_insensitive (editor, "palette-optionmenu",
-                   mask & TERMINAL_SETTING_PALETTE);
-
-  if (terminal_widget_supports_pango_fonts ())
-    set_insensitive (editor, "font-hbox",
-                     mask & TERMINAL_SETTING_FONT);
-  else    
-    set_insensitive (editor, "font-hbox",
-                     mask & TERMINAL_SETTING_X_FONT);
+                   mask->palette);
 
   set_insensitive (editor, "solid-radiobutton",
-                   mask & TERMINAL_SETTING_BACKGROUND_TYPE);
+                   mask->background_type);
   set_insensitive (editor, "image-radiobutton",
-                   mask & TERMINAL_SETTING_BACKGROUND_TYPE);
+                   mask->background_type);
   set_insensitive (editor, "transparent-radiobutton",
-                   mask & TERMINAL_SETTING_BACKGROUND_TYPE);
-
-  set_insensitive (editor, "darken-background-checkbutton",
-                   mask & TERMINAL_SETTING_BACKGROUND_DARKNESS);
+                   mask->background_type);
 
   set_insensitive (editor, "backspace-binding-optionmenu",
-                   mask & TERMINAL_SETTING_BACKSPACE_BINDING);
+                   mask->backspace_binding);
   set_insensitive (editor, "delete-binding-optionmenu",
-                   mask & TERMINAL_SETTING_DELETE_BINDING);
+                   mask->delete_binding);
 
   set_insensitive (editor, "use-theme-colors-checkbutton",
-                   mask & TERMINAL_SETTING_USE_THEME_COLORS);
+                   mask->use_theme_colors);
+
+  set_insensitive (editor, "system-font-checkbutton",
+                   mask->use_system_font);
+
   
   {
     int i;
@@ -1400,7 +1480,7 @@ profile_editor_update_sensitivity (GtkWidget       *editor,
       {
         char *s = g_strdup_printf ("palette-colorpicker-%d", i+1);
 
-        set_insensitive (editor, s, mask & TERMINAL_SETTING_PALETTE);
+        set_insensitive (editor, s, mask->palette);
 
         g_free (s);
 
@@ -1563,10 +1643,10 @@ profile_editor_update_silent_bell (GtkWidget       *editor,
 {
   GtkWidget *w;
 
-  w = profile_editor_get_widget (editor, "silent-bell-checkbutton");
+  w = profile_editor_get_widget (editor, "bell-checkbutton");
   
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-                                terminal_profile_get_silent_bell (profile));
+                                !terminal_profile_get_silent_bell (profile));
 }
 
 static void
@@ -1783,7 +1863,7 @@ profile_editor_update_x_font (GtkWidget       *editor,
   GtkWidget *fontsel;
   char *spacings[] = { "m", "c", NULL };
   char *slants[] = { "r", "ot", NULL };
-  char *weights[] = { "medium", "regular", NULL };
+  char *weights[] = { "medium", "regular", "demibold", NULL };
   gchar *name;
 
   if (terminal_widget_supports_pango_fonts ())
@@ -1875,12 +1955,11 @@ profile_editor_update_background_darkness (GtkWidget       *editor,
   GtkWidget *w;
   double v;
 
-  w = profile_editor_get_widget (editor, "darken-background-checkbutton");
+  w = profile_editor_get_widget (editor, "darken-background-scale");
 
   v = terminal_profile_get_background_darkness (profile);
   
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
-                                v >= DARKNESS_THRESHOLD);
+  gtk_range_set_value (GTK_RANGE (w), v);
 }
 
 static void
@@ -1925,12 +2004,10 @@ profile_editor_update_use_system_font (GtkWidget       *editor,
 {
   GtkWidget *w;
 
-#if 0
-  w = profile_editor_get_widget (editor, "use-system-font-checkbutton");
+  w = profile_editor_get_widget (editor, "system-font-checkbutton");
   
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
                                 terminal_profile_get_use_system_font (profile));
-#endif
 }
 
 static void
